@@ -1,6 +1,7 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 #
-# $Id: __init__.py 1237 2011-12-14 17:56:08Z g.rodola $
+# $Id: __init__.py 1381 2012-06-27 13:57:01Z g.rodola $
 #
 # Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -11,7 +12,9 @@ processes and gather system information in a portable way by using
 Python.
 """
 
-__version__ = "0.4.1"
+from __future__ import division
+
+__version__ = "0.5.0"
 version_info = tuple([int(num) for num in __version__.split('.')])
 
 __all__ = [
@@ -28,7 +31,7 @@ __all__ = [
     # functions
     "test", "pid_exists", "get_pid_list", "process_iter", "get_process_list",
     "phymem_usage", "virtmem_usage"
-    "cpu_times", "per_cpu_times", "cpu_percent", "per_cpu_percent",
+    "cpu_times", "cpu_percent", "per_cpu_percent",
     "network_io_counters", "disk_io_counters",
     ]
 
@@ -45,15 +48,17 @@ except ImportError:
     pwd = None
 
 from psutil.error import Error, NoSuchProcess, AccessDenied, TimeoutExpired
-from psutil._compat import property
+from psutil._compat import property, defaultdict
+from psutil._common import cached_property
+from psutil._common import (nt_disk_iostat as _nt_disk_iostat,
+                            nt_net_iostat as _nt_net_iostat)
 from psutil._common import (STATUS_RUNNING, STATUS_IDLE, STATUS_SLEEPING,
                             STATUS_DISK_SLEEP, STATUS_STOPPED,
                             STATUS_TRACING_STOP, STATUS_ZOMBIE, STATUS_DEAD,
-                            STATUS_WAKING, STATUS_LOCKED
-                            )
+                            STATUS_WAKING, STATUS_LOCKED)
 
 # import the appropriate module for our platform only
-if sys.platform.lower().startswith("linux"):
+if sys.platform.startswith("linux"):
     import psutil._pslinux as _psplatform
     from psutil._pslinux import (phymem_buffers,
                                  cached_phymem,
@@ -64,7 +69,7 @@ if sys.platform.lower().startswith("linux"):
     phymem_buffers = _psplatform.phymem_buffers
     cached_phymem = _psplatform.cached_phymem
 
-elif sys.platform.lower().startswith("win32"):
+elif sys.platform.startswith("win32"):
     import psutil._psmswindows as _psplatform
     from psutil._psmswindows import (ABOVE_NORMAL_PRIORITY_CLASS,
                                      BELOW_NORMAL_PRIORITY_CLASS,
@@ -73,10 +78,10 @@ elif sys.platform.lower().startswith("win32"):
                                      NORMAL_PRIORITY_CLASS,
                                      REALTIME_PRIORITY_CLASS)
 
-elif sys.platform.lower().startswith("darwin"):
+elif sys.platform.startswith("darwin"):
     import psutil._psosx as _psplatform
 
-elif sys.platform.lower().startswith("freebsd"):
+elif sys.platform.startswith("freebsd"):
     import psutil._psbsd as _psplatform
 
 else:
@@ -88,28 +93,28 @@ NUM_CPUS = _psplatform.NUM_CPUS
 BOOT_TIME = _psplatform.BOOT_TIME
 TOTAL_PHYMEM = _psplatform.phymem_usage()[0]
 
-get_pid_list = _psplatform.get_pid_list
-pid_exists = _psplatform.pid_exists
-
 
 class Process(object):
     """Represents an OS process."""
 
     def __init__(self, pid):
-        """Create a new Process object, raises NoSuchProcess if the PID
-        does not exist, and ValueError if the parameter is not an
-        integer PID.
+        """Create a new Process object for the given pid.
+        Raises NoSuchProcess if pid does not exist.
         """
-        if not isinstance(pid, int):
-            raise ValueError("an integer is required")
-        if not pid_exists(pid):
-            raise NoSuchProcess(pid, None, "no process found with pid %s" % pid)
         self._pid = pid
+        self._gone = False
         # platform-specific modules define an _psplatform.Process
         # implementation class
         self._platform_impl = _psplatform.Process(pid)
         self._last_sys_cpu_times = None
         self._last_proc_cpu_times = None
+        # cache creation time for later use in is_running() method
+        try:
+            self.create_time
+        except AccessDenied:
+            pass
+        except NoSuchProcess:
+            raise NoSuchProcess(pid, None, 'no process found with pid %s' % pid)
 
     def __str__(self):
         try:
@@ -127,12 +132,61 @@ class Process(object):
     def __repr__(self):
         return "<%s at %s>" % (self.__str__(), id(self))
 
+    def as_dict(self, attrs=[], ad_value=None):
+        """Utility method returning process information as a hashable
+        dictionary.
+
+        If 'attrs' is specified it must be a list of strings reflecting
+        available Process class's attribute names (e.g. ['get_cpu_times',
+        'name']) else all public (read only) attributes are assumed.
+
+        'ad_value' is the value which gets assigned to a dict key in case
+        AccessDenied exception is raised when retrieving that particular
+        process information.
+        """
+        excluded_names = set(['send_signal', 'suspend', 'resume', 'terminate',
+                              'kill', 'wait', 'is_running', 'as_dict', 'parent',
+                              'get_children', 'nice'])
+        retdict = dict()
+        for name in set(attrs or dir(self)):
+            if name.startswith('_'):
+                continue
+            if name.startswith('set_'):
+                continue
+            if name in excluded_names:
+                continue
+            try:
+                attr = getattr(self, name)
+                if callable(attr):
+                    if name == 'get_cpu_percent':
+                        ret = attr(interval=0)
+                    else:
+                        ret = attr()
+                else:
+                    ret = attr
+            except AccessDenied:
+                ret = ad_value
+            except NotImplementedError:
+                # in case of not implemented functionality (may happen
+                # on old or exotic systems) we want to crash only if
+                # the user explicitly asked for that particular attr
+                if attrs:
+                    raise
+                continue
+            if name.startswith('get'):
+                if name[3] == '_':
+                    name = name[4:]
+                elif name == 'getcwd':
+                    name = 'cwd'
+            retdict[name] = ret
+        return retdict
+
     @property
     def pid(self):
         """The process pid."""
         return self._pid
 
-    @property
+    @cached_property
     def ppid(self):
         """The process parent pid."""
         return self._platform_impl.get_process_ppid()
@@ -149,7 +203,7 @@ class Process(object):
             except NoSuchProcess:
                 pass
 
-    @property
+    @cached_property
     def name(self):
         """The process name."""
         name = self._platform_impl.get_process_name()
@@ -167,7 +221,7 @@ class Process(object):
         self._platform_impl._process_name = name
         return name
 
-    @property
+    @cached_property
     def exe(self):
         """The process executable as an absolute path name."""
         exe = self._platform_impl.get_process_exe()
@@ -182,7 +236,7 @@ class Process(object):
             raise AccessDenied(self.pid, self._platform_impl._process_name)
         return exe
 
-    @property
+    @cached_property
     def cmdline(self):
         """The command line process has been called with."""
         return self._platform_impl.get_process_cmdline()
@@ -192,16 +246,6 @@ class Process(object):
         """The process current status as a STATUS_* constant."""
         return self._platform_impl.get_process_status()
 
-    @property
-    def nice(self):
-        """Get or set process niceness (priority)."""
-        return self._platform_impl.get_process_nice()
-
-    @nice.setter
-    def nice(self, value):
-        # invoked on "p.nice = num"; change process niceness
-        return self._platform_impl.set_process_nice(value)
-
     if os.name == 'posix':
 
         @property
@@ -238,7 +282,7 @@ class Process(object):
         else:
             return self._platform_impl.get_process_username()
 
-    @property
+    @cached_property
     def create_time(self):
         """The process creation time as a floating point number
         expressed in seconds since the epoch, in UTC.
@@ -264,6 +308,14 @@ class Process(object):
             """
             return self._platform_impl.get_process_io_counters()
 
+    def get_nice(self):
+        """Get process niceness (priority)."""
+        return self._platform_impl.get_process_nice()
+
+    def set_nice(self, value):
+        """Set process niceness (priority)."""
+        return self._platform_impl.set_process_nice(value)
+
     # available only on Linux
     if hasattr(_psplatform.Process, "get_process_ionice"):
 
@@ -279,6 +331,36 @@ class Process(object):
             """
             return self._platform_impl.set_process_ionice(ioclass, value)
 
+    # available on Windows and Linux only
+    if hasattr(_psplatform.Process, "get_process_cpu_affinity"):
+
+        def get_cpu_affinity(self):
+            """Get process current CPU affinity."""
+            return self._platform_impl.get_process_cpu_affinity()
+
+        def set_cpu_affinity(self, cpus):
+            """Set process current CPU affinity.
+            'cpus' is a list of CPUs for which you want to set the
+            affinity (e.g. [0, 1]).
+            """
+            return self._platform_impl.set_process_cpu_affinity(cpus)
+
+    if os.name == 'nt':
+
+        def get_num_handles(self):
+            """Return the number of handles opened by this process
+            (Windows only).
+            """
+            return self._platform_impl.get_num_handles()
+
+    if os.name == 'posix':
+
+        def get_num_fds(self):
+            """Return the number of file descriptors opened by this
+            process (POSIX only).
+            """
+            return self._platform_impl.get_num_fds()
+
     def get_num_threads(self):
         """Return the number of threads used by this process."""
         return self._platform_impl.get_process_num_threads()
@@ -289,21 +371,62 @@ class Process(object):
         """
         return self._platform_impl.get_process_threads()
 
-    def get_children(self):
+    def get_children(self, recursive=False):
         """Return the children of this process as a list of Process
         objects.
+        If recursive is True return all the parent descendants.
+
+        Example (A == this process):
+
+         A ─┐
+            │
+            ├─ B (child) ─┐
+            │             └─ X (grandchild) ─┐
+            │                                └─ Y (great grandchild)
+            ├─ C (child)
+            └─ D (child)
+
+        >>> p.get_children()
+        B, C, D
+        >>> p.get_children(recursive=True)
+        B, X, Y, C, D
+
+        Note that in the example above if process X disappears
+        process Y won't be returned either as the reference to
+        process A is lost.
         """
         if not self.is_running():
             name = self._platform_impl._process_name
             raise NoSuchProcess(self.pid, name)
-        retlist = []
-        for proc in process_iter():
-            try:
-                if proc.ppid == self.pid:
-                    retlist.append(proc)
-            except NoSuchProcess:
-                pass
-        return retlist
+
+        ret = []
+        if not recursive:
+            for p in process_iter():
+                try:
+                    if p.ppid == self.pid:
+                        ret.append(p)
+                except NoSuchProcess:
+                    pass
+        else:
+            # construct a dict where 'values' are all the processes
+            # having 'key' as their parent
+            table = defaultdict(list)
+            for p in process_iter():
+                try:
+                    table[p.ppid].append(p)
+                except NoSuchProcess:
+                    pass
+            # At this point we have a mapping table where table[self.pid]
+            # are the current process's children.
+            # Below, we look for all descendants recursively, similarly
+            # to a recursive function call.
+            checkpids = [self.pid]
+            for pid in checkpids:
+                for proc in table[pid]:
+                    ret.append(proc)
+                    if proc.pid not in checkpids:
+                        checkpids.append(proc.pid)
+        return ret
 
     def get_cpu_percent(self, interval=0.1):
         """Return a float representing the current process CPU
@@ -384,6 +507,33 @@ class Process(object):
         except ZeroDivisionError:
             return 0.0
 
+    def get_memory_maps(self, grouped=True):
+        """Return process's mapped memory regions as a list of nameduples
+        whose fields are variable depending on the platform.
+
+        If 'grouped' is True the mapped regions with the same 'path'
+        are grouped together and the different memory fields are summed.
+
+        If 'grouped' is False every mapped region is shown as a single
+        entity and the namedtuple will also include the mapped region's
+        address space ('addr') and permission set ('perms').
+        """
+        it = self._platform_impl.get_memory_maps()
+        if grouped:
+            d = {}
+            for tupl in it:
+                path = tupl[2]
+                nums = tupl[3:]
+                try:
+                    d[path] = map(lambda x, y: x+y, d[path], nums)
+                except KeyError:
+                    d[path] = nums
+            nt = self._platform_impl.nt_mmap_grouped
+            return [nt(path, *d[path]) for path in d]
+        else:
+            nt = self._platform_impl.nt_mmap_ext
+            return [nt(*x) for x in it]
+
     def get_open_files(self):
         """Return files opened by process as a list of namedtuples
         including absolute file name and file descriptor number.
@@ -411,20 +561,18 @@ class Process(object):
 
     def is_running(self):
         """Return whether this process is running."""
+        if self._gone:
+            return False
         try:
-            # Test for equality with another Process object based
-            # on pid and creation time.
-            # This pair is supposed to indentify a Process instance
-            # univocally over the time (the PID alone is not enough as
-            # it might refer to a process which is gone in meantime
-            # and its PID reused by another process).
-            new_self = Process(self.pid)
-            p1 = (self.pid, self.create_time)
-            p2 = (new_self.pid, new_self.create_time)
+            # Checking if pid is alive is not enough as the pid might
+            # have been reused by another process.
+            # pid + creation time, on the other hand, is supposed to
+            # identify a process univocally.
+            return self.create_time == \
+                   self._platform_impl.get_process_create_time()
         except NoSuchProcess:
+            self._gone = True
             return False
-        else:
-            return p1 == p2
 
     def send_signal(self, sig):
         """Send a signal to process (see signal module constants).
@@ -439,7 +587,8 @@ class Process(object):
         if os.name == 'posix':
             try:
                 os.kill(self.pid, sig)
-            except OSError, err:
+            except OSError:
+                err = sys.exc_info()[1]
                 name = self._platform_impl._process_name
                 if err.errno == errno.ESRCH:
                     raise NoSuchProcess(self.pid, name)
@@ -506,12 +655,31 @@ class Process(object):
             raise ValueError("timeout must be a positive integer")
         return self._platform_impl.process_wait(timeout)
 
+    # --- deprecated API
+
+    @property
+    def nice(self):
+        """Get or set process niceness (priority).
+        Deprecated, use get_nice() instead.
+        """
+        msg = "this property is deprecated; use Process.get_nice() method instead"
+        warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
+        return self.get_nice()
+
+    @nice.setter
+    def nice(self, value):
+        # invoked on "p.nice = num"; change process niceness
+        # deprecated in favor of set_nice()
+        msg = "this property is deprecated; use Process.set_nice() method instead"
+        warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
+        return self.set_nice(value)
+
 
 class Popen(Process):
     """A more convenient interface to stdlib subprocess module.
     It starts a sub process and deals with it exactly as when using
     subprocess.Popen class but in addition also provides all the
-    property and methods of psutil.Process class in a unique interface:
+    property and methods of psutil.Process class in a single interface:
 
       >>> import psutil
       >>> from subprocess import PIPE
@@ -538,7 +706,18 @@ class Popen(Process):
 
     def __init__(self, *args, **kwargs):
         self.__subproc = subprocess.Popen(*args, **kwargs)
-        Process.__init__(self, self.__subproc.pid)
+        self._pid = self.__subproc.pid
+        self._gone = False
+        self._platform_impl = _psplatform.Process(self._pid)
+        self._last_sys_cpu_times = None
+        self._last_proc_cpu_times = None
+        try:
+            self.create_time
+        except AccessDenied:
+            pass
+        except NoSuchProcess:
+            raise NoSuchProcess(self._pid, None,
+                                "no process found with pid %s" % pid)
 
     def __dir__(self):
         return list(set(dir(Popen) + dir(subprocess.Popen)))
@@ -553,22 +732,54 @@ class Popen(Process):
                 raise AttributeError("%s instance has no attribute '%s'"
                                       %(self.__class__.__name__, name))
 
-def process_iter():
-    """Return an iterator yielding a Process class instances for all
-    running processes on the local machine.
-    """
-    pids = get_pid_list()
-    for pid in pids:
-        try:
-            yield Process(pid)
-        except (NoSuchProcess, AccessDenied):
-            continue
 
-def get_process_list():
-    """Return a list of Process class instances for all running
-    processes on the local machine.
+get_pid_list = _psplatform.get_pid_list
+pid_exists = _psplatform.pid_exists
+
+_pmap = {}
+
+def process_iter():
+    """Return a generator yielding a Process class instance for all
+    running processes on the local machine.
+
+    Every new Process instance is only created once and then cached
+    into an internal table which is updated every time this is used.
     """
-    return list(process_iter())
+    def add(pid):
+        proc = Process(pid)
+        _pmap[proc.pid] = proc
+        return proc
+
+    def remove(pid):
+        _pmap.pop(pid, None)
+
+    a = set(get_pid_list())
+    b = set(_pmap.keys())
+    new_pids = a - b
+    gone_pids = b - a
+
+    for pid in gone_pids:
+        remove(pid)
+    for pid, proc in list(_pmap.items()):
+        try:
+            # use is_running() to check whether PID has been reused by
+            # another process in which case yield a new Process instance
+            if proc.is_running():
+                yield proc
+            else:
+                yield add(pid)
+        except NoSuchProcess:
+            remove(pid)
+        except AccessDenied:
+            # Process creation time can't be determined hence there's
+            # no way to tell whether the pid of the cached process
+            # has been reused. Just return the cached version.
+            yield proc
+    for pid in new_pids:
+        try:
+            yield add(pid)
+        except NoSuchProcess:
+            pass
 
 def cpu_times(percpu=False):
     """Return system-wide CPU times as a namedtuple object.
@@ -683,7 +894,8 @@ def disk_usage(path):
 
 def disk_partitions(all=False):
     """Return mounted partitions as a list of namedtuples including
-    device, mount point and filesystem type.
+    device, mount point, filesystem type and mount options (a raw
+    string separated by commas which may vary depending on the platform).
 
     If "all" parameter is False return physical devices only and ignore
     all others.
@@ -704,21 +916,15 @@ def network_io_counters(pernic=False):
     with network interface names as the keys and the namedtuple
     described above as the values.
     """
-    from psutil._common import ntuple_net_iostat
     rawdict = _psplatform.network_io_counters()
+    if not rawdict:
+        raise RuntimeError("couldn't find any network interface")
     if pernic:
-        for nic, fields in rawdict.iteritems():
-            rawdict[nic] = ntuple_net_iostat(*fields)
+        for nic, fields in rawdict.items():
+            rawdict[nic] = _nt_net_iostat(*fields)
         return rawdict
     else:
-        bytes_sent, bytes_recv, packets_sent, packets_recv = 0, 0, 0, 0
-        for _, fields in rawdict.iteritems():
-            bytes_sent += fields[0]
-            bytes_recv += fields[1]
-            packets_sent += fields[2]
-            packets_recv += fields[3]
-        return ntuple_net_iostat(bytes_sent, bytes_recv,
-                                 packets_sent, packets_recv)
+        return _nt_net_iostat(*[sum(x) for x in zip(*rawdict.values())])
 
 def disk_io_counters(perdisk=False):
     """Return system disk I/O statistics as a namedtuple including
@@ -736,30 +942,36 @@ def disk_io_counters(perdisk=False):
     with partition names as the keys and the namedutuple
     described above as the values.
     """
-    from psutil._common import ntuple_disk_iostat
     rawdict = _psplatform.disk_io_counters()
+    if not rawdict:
+        raise RuntimeError("couldn't find any physical disk")
     if perdisk:
-        for disk, fields in rawdict.iteritems():
-            rawdict[disk] = ntuple_disk_iostat(*fields)
+        for disk, fields in rawdict.items():
+            rawdict[disk] = _nt_disk_iostat(*fields)
         return rawdict
     else:
-        reads, writes, rbytes, wbytes, rtime, wtime = 0, 0, 0, 0, 0, 0
-        for _, fields in rawdict.iteritems():
-            reads += fields[0]
-            writes += fields[1]
-            rbytes += fields[2]
-            wbytes += fields[3]
-            rtime += fields[4]
-            wtime += fields[5]
-        return ntuple_disk_iostat(reads, writes, rbytes, wbytes, rtime, wtime)
+        return _nt_disk_iostat(*[sum(x) for x in zip(*rawdict.values())])
 
+def get_users():
+    """Return users currently connected on the system as a list of
+    namedtuples including the following attributes.
 
-def _deprecated(replacement):
-    # a decorator which can be used to mark functions as deprecated
+     - user: the name of the user
+     - terminal: the tty or pseudo-tty associated with the user, if any.
+     - host: the host name associated with the entry, if any.
+     - started: the creation time as a floating point number expressed in
+       seconds since the epoch.
+    """
+    return _psplatform.get_system_users()
+
+# http://goo.gl/jYLvf
+def _deprecated(replacement=None):
+    """A decorator which can be used to mark functions as deprecated."""
     def outer(fun):
         def inner(*args, **kwargs):
-            msg = "psutil.%s is deprecated; use %s instead" \
-                  % (fun.__name__, replacement)
+            msg = "psutil.%s is deprecated" % fun.__name__
+            if replacement is not None:
+                msg += "; use %s instead" % replacement
             warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
             return fun(*args, **kwargs)
         return inner
@@ -767,6 +979,13 @@ def _deprecated(replacement):
 
 # --- deprecated functions
 
+@_deprecated()
+def get_process_list():
+    """Return a list of Process class instances for all running
+    processes on the local machine (deprecated).
+    """
+    return list(process_iter())
+
 @_deprecated("psutil.phymem_usage")
 def avail_phymem():
     return phymem_usage().free
@@ -788,49 +1007,56 @@ def avail_virtmem():
     return virtmem_usage().free
 
 def test():
-    """List info of all currently running processes emulating a
-    ps -aux output.
+    """List info of all currently running processes emulating ps aux
+    output.
     """
     import datetime
+    from psutil._compat import print_
+
     today_day = datetime.date.today()
-
-    def get_process_info(pid):
-        proc = Process(pid)
-        user = proc.username
-        if os.name == 'nt' and '\\' in user:
-            user = user.split('\\')[1]
-        pid = proc.pid
-        cpu = round(proc.get_cpu_percent(interval=None), 1)
-        mem = round(proc.get_memory_percent(), 1)
-        rss, vsz = [x / 1024 for x in proc.get_memory_info()]
-
-        # If process has been created today print H:M, else MonthDay
-        start = datetime.datetime.fromtimestamp(proc.create_time)
-        if start.date() == today_day:
-            start = start.strftime("%H:%M")
-        else:
-            start = start.strftime("%b%d")
-
-        cputime = time.strftime("%M:%S", time.localtime(sum(proc.get_cpu_times())))
-        cmd = ' '.join(proc.cmdline)
-        # where cmdline is not available UNIX shows process name between
-        # [] parentheses
-        if not cmd:
-            cmd = "[%s]" % proc.name
-        return "%-9s %-5s %-4s %4s %7s %7s %5s %8s %s" \
-                % (user, pid, cpu, mem, vsz, rss, start, cputime, cmd)
-
-    print "%-9s %-5s %-4s %4s %7s %7s %5s %7s  %s" \
-      % ("USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "START", "TIME", "COMMAND")
-    pids = get_pid_list()
-    pids.sort()
-    for pid in pids:
+    templ = "%-10s %5s %4s %4s %7s %7s %-13s %5s %7s  %s"
+    attrs = ['pid', 'username', 'get_cpu_percent', 'get_memory_percent', 'name',
+             'get_cpu_times', 'create_time', 'get_memory_info']
+    if os.name == 'posix':
+        attrs.append('terminal')
+    print_(templ % ("USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY", "START",
+                    "TIME", "COMMAND"))
+    for p in sorted(process_iter(), key=lambda p: p.pid):
         try:
-            line = get_process_info(pid)
-        except (AccessDenied, NoSuchProcess):
+            pinfo = p.as_dict(attrs, ad_value='')
+        except NoSuchProcess:
             pass
         else:
-            print line
+            if pinfo['create_time']:
+                ctime = datetime.datetime.fromtimestamp(pinfo['create_time'])
+                if ctime.date() == today_day:
+                    ctime = ctime.strftime("%H:%M")
+                else:
+                    ctime = ctime.strftime("%b%d")
+            cputime = time.strftime("%M:%S", time.localtime(sum(pinfo['cpu_times'])))
+            user = pinfo['username']
+            if os.name == 'nt' and '\\' in user:
+                user = user.split('\\')[1]
+            vms = pinfo['memory_info'] and \
+                  int(pinfo['memory_info'].vms / 1024) or '?'
+            rss = pinfo['memory_info'] and \
+                  int(pinfo['memory_info'].rss / 1024) or '?'
+            memp = pinfo['memory_percent'] and \
+                   round(pinfo['memory_percent'], 1) or '?'
+            print_(templ % (user[:10],
+                            pinfo['pid'],
+                            pinfo['cpu_percent'],
+                            memp,
+                            vms,
+                            rss,
+                            pinfo.get('terminal', '') or '?',
+                            ctime,
+                            cputime,
+                            pinfo['name'].strip() or '?'))
 
 if __name__ == "__main__":
     test()
+
+del property, cached_property, division
+if sys.version_info < (3, 0):
+    del num
