@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: _pslinux.py 1237 2011-12-14 17:56:08Z g.rodola $
+# $Id: _pslinux.py 1393 2012-06-28 15:26:27Z g.rodola $
 #
 # Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -8,6 +8,8 @@
 
 """Linux platform implementation."""
 
+from __future__ import division
+
 import os
 import errno
 import socket
@@ -21,6 +23,7 @@ import _psutil_linux
 from psutil import _psposix
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._common import *
+from psutil._compat import PY3, xrange, long
 
 __extra__all__ = [
     "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE",
@@ -80,6 +83,7 @@ def _get_num_cpus():
 
 # Number of clock ticks per second
 _CLOCK_TICKS = os.sysconf(os.sysconf_names["SC_CLK_TCK"])
+_PAGESIZE = os.sysconf("SC_PAGE_SIZE")
 _TERMINAL_MAP = _psposix._get_terminal_map()
 BOOT_TIME = _get_boot_time()
 NUM_CPUS = _get_num_cpus()
@@ -136,7 +140,7 @@ def phymem_usage():
     cached = cached_phymem()
     used = total - free
     percent = usage_percent(total - (free + buffers + cached), total, _round=1)
-    return ntuple_sysmeminfo(total, used, free, percent)
+    return nt_sysmeminfo(total, used, free, percent)
 
 
 def virtmem_usage():
@@ -153,7 +157,7 @@ def virtmem_usage():
         assert total is not None and free is not None
         used = total - free
         percent = usage_percent(used, total, _round=1)
-        return ntuple_sysmeminfo(total, used, free, percent)
+        return nt_sysmeminfo(total, used, free, percent)
     finally:
         f.close()
 
@@ -172,7 +176,7 @@ def get_system_cpu_times():
 
     values = values[1:8]
     values = tuple([float(x) / _CLOCK_TICKS for x in values])
-    return ntuple_sys_cputimes(*values[:7])
+    return nt_sys_cputimes(*values[:7])
 
 def get_system_per_cpu_times():
     """Return a list of namedtuple representing the CPU times
@@ -187,7 +191,7 @@ def get_system_per_cpu_times():
             if line.startswith('cpu'):
                 values = line.split()[1:8]
                 values = tuple([float(x) / _CLOCK_TICKS for x in values])
-                entry = ntuple_sys_cputimes(*values[:7])
+                entry = nt_sys_cputimes(*values[:7])
                 cpus.append(entry)
         return cpus
     finally:
@@ -210,19 +214,38 @@ def disk_partitions(all=False):
     retlist = []
     partitions = _psutil_linux.get_disk_partitions()
     for partition in partitions:
-        device, mountpoint, fstype = partition
+        device, mountpoint, fstype, opts = partition
         if device == 'none':
             device = ''
         if not all:
             if device == '' or fstype not in phydevs:
                 continue
-        ntuple = ntuple_partition(device, mountpoint, fstype)
+        ntuple = nt_partition(device, mountpoint, fstype, opts)
         retlist.append(ntuple)
     return retlist
 
 get_disk_usage = _psposix.get_disk_usage
 
 
+# --- other sysetm functions
+
+def get_system_users():
+    """Return currently connected users as a list of namedtuples."""
+    retlist = []
+    rawlist = _psutil_linux.get_system_users()
+    for item in rawlist:
+        user, tty, hostname, tstamp, user_process = item
+        # XXX the underlying C function includes entries about
+        # system boot, run level and others.  We might want
+        # to use them in the future.
+        if not user_process:
+            continue
+        if hostname == ':0.0':
+            hostname = 'localhost'
+        nt = nt_user(user, tty or None, hostname, tstamp)
+        retlist.append(nt)
+    return retlist
+
 # --- process functions
 
 def get_pid_list():
@@ -246,12 +269,14 @@ def network_io_counters():
 
     retdict = {}
     for line in lines[2:]:
-        fields = line.split()
-        name = fields[0][:-1]
-        bytes_recv = int(fields[1])
-        packets_recv = int(fields[2])
-        bytes_sent = int(fields[9])
-        packets_sent = int(fields[10])
+        colon = line.find(':')
+        assert colon > 0, line
+        name = line[:colon].strip()
+        fields = line[colon+1:].strip().split()
+        bytes_recv = int(fields[0])
+        packets_recv = int(fields[1])
+        bytes_sent = int(fields[8])
+        packets_sent = int(fields[9])
         retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv)
     return retdict
 
@@ -319,10 +344,11 @@ def wrap_exceptions(callable):
     def wrapper(self, *args, **kwargs):
         try:
             return callable(self, *args, **kwargs)
-        except (OSError, IOError), err:
+        except EnvironmentError:
             # ENOENT (no such file or directory) gets raised on open().
             # ESRCH (no such process) can get raised on read() if
             # process is gone in meantime.
+            err = sys.exc_info()[1]
             if err.errno in (errno.ENOENT, errno.ESRCH):
                 raise NoSuchProcess(self.pid, self._process_name)
             if err.errno in (errno.EPERM, errno.EACCES):
@@ -337,6 +363,8 @@ class Process(object):
     __slots__ = ["pid", "_process_name"]
 
     def __init__(self, pid):
+        if not isinstance(pid, int):
+            raise TypeError('pid must be an integer')
         self.pid = pid
         self._process_name = None
 
@@ -355,7 +383,8 @@ class Process(object):
             raise AccessDenied(self.pid, self._process_name)
         try:
             exe = os.readlink("/proc/%s/exe" % self.pid)
-        except (OSError, IOError), err:
+        except (OSError, IOError):
+            err = sys.exc_info()[1]
             if err.errno == errno.ENOENT:
                 # no such file error; might be raised also if the
                 # path actually exists for system processes with
@@ -416,10 +445,14 @@ class Process(object):
                     read_bytes = int(line.split()[1])
                 elif line.startswith("write_bytes"):
                     write_bytes = int(line.split()[1])
-            return ntuple_io(read_count, write_count, read_bytes, write_bytes)
+            return nt_io(read_count, write_count, read_bytes, write_bytes)
         finally:
             f.close()
 
+    if not os.path.exists('/proc/%s/io' % os.getpid()):
+        def get_process_io_counters(self):
+            raise NotImplementedError('/proc/PID/io is not available')
+
     @wrap_exceptions
     def get_cpu_times(self):
         f = open("/proc/%s/stat" % self.pid)
@@ -432,7 +465,7 @@ class Process(object):
         values = st.split(' ')
         utime = float(values[11]) / _CLOCK_TICKS
         stime = float(values[12]) / _CLOCK_TICKS
-        return ntuple_cputimes(utime, stime)
+        return nt_cputimes(utime, stime)
 
     @wrap_exceptions
     def process_wait(self, timeout=None):
@@ -449,7 +482,7 @@ class Process(object):
         finally:
             f.close()
         # ignore the first two values ("pid (exe)")
-        st = st[st.find(')') + 2:]
+        st = st[st.rfind(')') + 2:]
         values = st.split(' ')
         # According to documentation, starttime is in field 21 and the
         # unit is jiffies (clock ticks).
@@ -460,22 +493,87 @@ class Process(object):
 
     @wrap_exceptions
     def get_memory_info(self):
-        f = open("/proc/%s/status" % self.pid)
+        f = open("/proc/%s/statm" % self.pid)
         try:
-            virtual_size = 0
-            resident_size = 0
-            _flag = False
-            for line in f:
-                if (not _flag) and line.startswith("VmSize:"):
-                    virtual_size = int(line.split()[1]) * 1024
-                    _flag = True
-                elif line.startswith("VmRSS"):
-                    resident_size = int(line.split()[1]) * 1024
-                    break
-            return ntuple_meminfo(resident_size, virtual_size)
+            vms, rss = f.readline().split()[:2]
+            return nt_meminfo(int(rss) * _PAGESIZE,
+                              int(vms) * _PAGESIZE)
         finally:
             f.close()
 
+    _mmap_base_fields = ['path', 'rss', 'size', 'pss', 'shared_clean',
+                         'shared_dirty', 'private_clean', 'private_dirty',
+                         'referenced', 'anonymous', 'swap',]
+    nt_mmap_grouped = namedtuple('mmap', ' '.join(_mmap_base_fields))
+    nt_mmap_ext = namedtuple('mmap', 'addr perms ' + ' '.join(_mmap_base_fields))
+
+    def get_memory_maps(self):
+        """Return process's mapped memory regions as a list of nameduples.
+        Fields are explained in 'man proc'; here is an updated (Apr 2012)
+        version: http://goo.gl/fmebo
+        """
+        f = None
+        try:
+            f = open("/proc/%s/smaps" % self.pid)
+            first_line = f.readline()
+            current_block = [first_line]
+
+            def get_blocks():
+                data = {}
+                for line in f:
+                    fields = line.split(None, 5)
+                    if len(fields) >= 5:
+                        yield (current_block.pop(), data)
+                        current_block.append(line)
+                    else:
+                        data[fields[0]] = int(fields[1]) * 1024
+                yield (current_block.pop(), data)
+
+            if first_line:  # smaps file can be empty
+                for header, data in get_blocks():
+                    hfields = header.split(None, 5)
+                    try:
+                        addr, perms, offset, dev, inode, path = hfields
+                    except ValueError:
+                        addr, perms, offset, dev, inode, path = hfields + ['']
+                    if not path:
+                        path = '[anon]'
+                    else:
+                        path = path.strip()
+                    yield (addr, perms, path,
+                           data['Rss:'],
+                           data['Size:'],
+                           data.get('Pss:', 0),
+                           data['Shared_Clean:'], data['Shared_Clean:'],
+                           data['Private_Clean:'], data['Private_Dirty:'],
+                           data['Referenced:'],
+                           data['Anonymous:'],
+                           data['Swap:'])
+            f.close()
+        except EnvironmentError:
+            # XXX - Can't use wrap_exceptions decorator as we're
+            # returning a generator;  this probably needs some
+            # refactoring in order to avoid this code duplication.
+            if f is not None:
+                f.close()
+            err = sys.exc_info()[1]
+            if err.errno in (errno.ENOENT, errno.ESRCH):
+                raise NoSuchProcess(self.pid, self._process_name)
+            if err.errno in (errno.EPERM, errno.EACCES):
+                raise AccessDenied(self.pid, self._process_name)
+            raise
+        except:
+            if f is not None:
+                f.close()
+            raise
+
+    if not os.path.exists('/proc/%s/smaps' % os.getpid()):
+        def get_shared_libs(self, ext):
+            msg = "this Linux version does not support /proc/PID/smaps " \
+                  "(kernel < 2.6.14 or CONFIG_MMU kernel configuration " \
+                  "option is not enabled)"
+            raise NotImplementedError(msg)
+
     @wrap_exceptions
     def get_process_cwd(self):
         # readlink() might return paths containing null bytes causing
@@ -500,13 +598,16 @@ class Process(object):
         thread_ids = os.listdir("/proc/%s/task" % self.pid)
         thread_ids.sort()
         retlist = []
+        hit_enoent = False
         for thread_id in thread_ids:
             try:
                 f = open("/proc/%s/task/%s/stat" % (self.pid, thread_id))
-            except (OSError, IOError), err:
+            except EnvironmentError:
+                err = sys.exc_info()[1]
                 if err.errno == errno.ENOENT:
                     # no such file or directory; it means thread
                     # disappeared on us
+                    hit_enoent = True
                     continue
                 raise
             try:
@@ -518,8 +619,11 @@ class Process(object):
             values = st.split(' ')
             utime = float(values[11]) / _CLOCK_TICKS
             stime = float(values[12]) / _CLOCK_TICKS
-            ntuple = ntuple_thread(int(thread_id), utime, stime)
+            ntuple = nt_thread(int(thread_id), utime, stime)
             retlist.append(ntuple)
+        if hit_enoent:
+            # raise NSP if the process disappeared on us
+            os.stat('/proc/%s' % self.pid)
         return retlist
 
     @wrap_exceptions
@@ -538,13 +642,43 @@ class Process(object):
     def set_process_nice(self, value):
         return _psutil_posix.setpriority(self.pid, value)
 
+    @wrap_exceptions
+    def get_process_cpu_affinity(self):
+        from_bitmask = lambda x: [i for i in xrange(64) if (1 << i) & x]
+        bitmask = _psutil_linux.get_process_cpu_affinity(self.pid)
+        return from_bitmask(bitmask)
+
+    @wrap_exceptions
+    def set_process_cpu_affinity(self, value):
+        def to_bitmask(l):
+            if not l:
+                raise ValueError("invalid argument %r" % l)
+            out = 0
+            for b in l:
+                if not isinstance(b, (int, long)) or b < 0:
+                    raise ValueError("invalid argument %r" % b)
+                out |= 2**b
+            return out
+
+        bitmask = to_bitmask(value)
+        try:
+            _psutil_linux.set_process_cpu_affinity(self.pid, bitmask)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno == errno.EINVAL:
+                allcpus = list(range(len(get_system_per_cpu_times())))
+                for cpu in value:
+                    if cpu not in allcpus:
+                        raise ValueError("invalid CPU %i" % cpu)
+            raise
+
     # only starting from kernel 2.6.13
     if hasattr(_psutil_linux, "ioprio_get"):
 
         @wrap_exceptions
         def get_process_ionice(self):
             ioclass, value = _psutil_linux.ioprio_get(self.pid)
-            return ntuple_ionice(ioclass, value)
+            return nt_ionice(ioclass, value)
 
         @wrap_exceptions
         def set_process_ionice(self, ioclass, value):
@@ -583,19 +717,30 @@ class Process(object):
     def get_open_files(self):
         retlist = []
         files = os.listdir("/proc/%s/fd" % self.pid)
+        hit_enoent = False
         for fd in files:
             file = "/proc/%s/fd/%s" % (self.pid, fd)
             if os.path.islink(file):
-                file = os.readlink(file)
-                if file.startswith("socket:["):
-                    continue
-                if file.startswith("pipe:["):
-                    continue
-                if file == "[]":
-                    continue
-                if os.path.isfile(file) and not file in retlist:
-                    ntuple = ntuple_openfile(file, int(fd))
-                    retlist.append(ntuple)
+                try:
+                    file = os.readlink(file)
+                except OSError:
+                    # ENOENT == file which is gone in the meantime
+                    err = sys.exc_info()[1]
+                    if err.errno == errno.ENOENT:
+                        hit_enoent = True
+                        continue
+                    raise
+                else:
+                    # If file is not an absolute path there's no way
+                    # to tell whether it's a regular file or not,
+                    # so we skip it. A regular file is always supposed
+                    # to be absolutized though.
+                    if file.startswith('/') and os.path.isfile(file):
+                        ntuple = nt_openfile(file, int(fd))
+                        retlist.append(ntuple)
+        if hit_enoent:
+            # raise NSP if the process disappeared on us
+            os.stat('/proc/%s' % self.pid)
         return retlist
 
     @wrap_exceptions
@@ -638,8 +783,9 @@ class Process(object):
             retlist = []
             try:
                 f = open(file, 'r')
-            except IOError, err:
+            except IOError:
                 # IPv6 not supported on this platform
+                err = sys.exc_info()[1]
                 if err.errno == errno.ENOENT and file.endswith('6'):
                     return []
                 else:
@@ -659,8 +805,8 @@ class Process(object):
                             else:
                                 status = ""
                             fd = int(inodes[inode])
-                            conn = ntuple_connection(fd, family, type_, laddr,
-                                                     raddr, status)
+                            conn = nt_connection(fd, family, type_, laddr,
+                                                 raddr, status)
                             retlist.append(conn)
                     else:
                         raise ValueError(family)
@@ -691,6 +837,8 @@ class Process(object):
         ret = []
         for f, family, type_ in tmap[kind]:
             ret += process("/proc/net/%s" % f, family, type_)
+        # raise NSP if the process disappeared on us
+        os.stat('/proc/%s' % self.pid)
         return ret
 
 
@@ -700,6 +848,10 @@ class Process(object):
 #        lsof = _psposix.LsofParser(self.pid, self._process_name)
 #        return lsof.get_process_connections()
 
+    @wrap_exceptions
+    def get_num_fds(self):
+       return len(os.listdir("/proc/%s/fd" % self.pid))
+
     @wrap_exceptions
     def get_process_ppid(self):
         f = open("/proc/%s/status" % self.pid)
@@ -719,7 +871,7 @@ class Process(object):
             for line in f:
                 if line.startswith('Uid:'):
                     _, real, effective, saved, fs = line.split()
-                    return ntuple_uids(int(real), int(effective), int(saved))
+                    return nt_uids(int(real), int(effective), int(saved))
             raise RuntimeError("line not found")
         finally:
             f.close()
@@ -731,7 +883,7 @@ class Process(object):
             for line in f:
                 if line.startswith('Gid:'):
                     _, real, effective, saved, fs = line.split()
-                    return ntuple_gids(int(real), int(effective), int(saved))
+                    return nt_gids(int(real), int(effective), int(saved))
             raise RuntimeError("line not found")
         finally:
             f.close()
@@ -755,7 +907,7 @@ class Process(object):
         """
         ip, port = addr.split(':')
         port = int(port, 16)
-        if sys.version_info >= (3,):
+        if PY3:
             ip = ip.encode('ascii')
         # this usually refers to a local socket in listen mode with
         # no end-points connected
