@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: _psmswindows.py 1157 2011-10-14 18:36:03Z g.rodola $
+# $Id: _psmswindows.py 1397 2012-06-29 16:33:08Z g.rodola $
 #
 # Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -15,8 +15,8 @@ import platform
 
 import _psutil_mswindows
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
-from psutil._compat import namedtuple
 from psutil._common import *
+from psutil._compat import PY3, xrange, long
 
 # Windows specific extended namespace
 __extra__all__ = ["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
@@ -28,7 +28,6 @@ __extra__all__ = ["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
 
 NUM_CPUS = _psutil_mswindows.get_num_cpus()
 BOOT_TIME = _psutil_mswindows.get_system_uptime()
-_WIN2000 = platform.win32_ver()[0] == '2000'
 ERROR_ACCESS_DENIED = 5
 WAIT_TIMEOUT = 0x00000102 # 258 in decimal
 
@@ -42,6 +41,21 @@ from _psutil_mswindows import (ABOVE_NORMAL_PRIORITY_CLASS,
                                REALTIME_PRIORITY_CLASS,
                                INFINITE)
 
+@memoize
+def _win32_QueryDosDevice(s):
+    return _psutil_mswindows.win32_QueryDosDevice(s)
+
+def _convert_raw_path(s):
+    # convert paths using native DOS format like:
+    # "\Device\HarddiskVolume1\Windows\systemew\file.txt"
+    # into: "C:\Windows\systemew\file.txt"
+    if PY3 and not isinstance(s, str):
+        s = s.decode('utf8')
+    rawdrive = '\\'.join(s.split('\\')[:3])
+    driveletter = _win32_QueryDosDevice(rawdrive)
+    return os.path.join(driveletter, s[len(rawdrive):])
+
+
 # --- public functions
 
 def phymem_usage():
@@ -49,7 +63,7 @@ def phymem_usage():
     all = _psutil_mswindows.get_system_phymem()
     total, free, total_pagef, avail_pagef, total_virt, free_virt, percent = all
     used = total - free
-    return ntuple_sysmeminfo(total, used, free, round(percent, 1))
+    return nt_sysmeminfo(total, used, free, round(percent, 1))
 
 def virtmem_usage():
     """Virtual system memory as a (total, used, free) tuple."""
@@ -58,38 +72,25 @@ def virtmem_usage():
     free_virt = all[5]
     used = total_virt - free_virt
     percent = usage_percent(used, total_virt, _round=1)
-    return ntuple_sysmeminfo(total_virt, used, free_virt, percent)
+    return nt_sysmeminfo(total_virt, used, free_virt, percent)
 
 def get_disk_usage(path):
     """Return disk usage associated with path."""
     try:
         total, free = _psutil_mswindows.get_disk_usage(path)
-    except WindowsError, err:
+    except WindowsError:
+        err = sys.exc_info()[1]
         if not os.path.exists(path):
             raise OSError(errno.ENOENT, "No such file or directory: '%s'" % path)
         raise
     used = total - free
     percent = usage_percent(used, total, _round=1)
-    return ntuple_diskinfo(total, used, free, percent)
+    return nt_diskinfo(total, used, free, percent)
 
 def disk_partitions(all):
     """Return disk partitions."""
-    retlist = []
-    drive_letters = _psutil_mswindows.win32_GetLogicalDriveStrings()
-    for letter in drive_letters:
-        mountpoint = letter
-        type = _psutil_mswindows.win32_GetDriveType(letter)
-        if not os.path.exists(mountpoint):
-            # usually a CD-ROM device with no disk inserted
-            mountpoint = ""
-        if not all:
-            if type not in ('cdrom', 'fixed', 'removable'):
-                continue
-            if not mountpoint:
-                continue
-        ntuple = ntuple_partition(letter, mountpoint, type)
-        retlist.append(ntuple)
-    return retlist
+    rawlist = _psutil_mswindows.get_disk_partitions(all)
+    return [nt_partition(*x) for x in rawlist]
 
 
 _cputimes_ntuple = namedtuple('cputimes', 'user system idle')
@@ -113,6 +114,16 @@ def get_system_per_cpu_times():
         ret.append(item)
     return ret
 
+def get_system_users():
+    """Return currently connected users as a list of namedtuples."""
+    retlist = []
+    rawlist = _psutil_mswindows.get_system_users()
+    for item in rawlist:
+        user, hostname, tstamp = item
+        nt = nt_user(user, None, hostname, tstamp)
+        retlist.append(nt)
+    return retlist
+
 get_pid_list = _psutil_mswindows.get_pid_list
 pid_exists = _psutil_mswindows.pid_exists
 network_io_counters = _psutil_mswindows.get_network_io_counters
@@ -128,7 +139,8 @@ def wrap_exceptions(callable):
     def wrapper(self, *args, **kwargs):
         try:
             return callable(self, *args, **kwargs)
-        except OSError, err:
+        except OSError:
+            err = sys.exc_info()[1]
             if err.errno in (errno.EPERM, errno.EACCES, ERROR_ACCESS_DENIED):
                 raise AccessDenied(self.pid, self._process_name)
             if err.errno == errno.ESRCH:
@@ -152,14 +164,16 @@ class Process(object):
         """Return process name as a string of limited len (15)."""
         return _psutil_mswindows.get_process_name(self.pid)
 
+    @wrap_exceptions
     def get_process_exe(self):
-        # no such thing as "exe" on Windows; it will maybe be determined
-        # later from cmdline[0]
-        if not pid_exists(self.pid):
-            raise NoSuchProcess(self.pid, self._process_name)
-        if self.pid in (0, 4):
-            raise AccessDenied(self.pid, self._process_name)
-        return ""
+        exe = _convert_raw_path(_psutil_mswindows.get_process_exe(self.pid))
+        # Note: if the folder containing the executable is changed
+        # GetProcessImageFileName() returns the previous name, hence
+        # this check. An empty string signals to try to guess the exe
+        # from cmdline later.
+        if not os.path.exists(exe):
+            exe = ""
+        return exe
 
     @wrap_exceptions
     def get_process_cmdline(self):
@@ -176,9 +190,30 @@ class Process(object):
         """Returns a tuple or RSS/VMS memory usage in bytes."""
         # special case for 0 (kernel processes) PID
         if self.pid == 0:
-            return ntuple_meminfo(0, 0)
+            return nt_meminfo(0, 0)
         rss, vms = _psutil_mswindows.get_memory_info(self.pid)
-        return ntuple_meminfo(rss, vms)
+        return nt_meminfo(rss, vms)
+
+    nt_mmap_grouped = namedtuple('mmap', 'path rss')
+    nt_mmap_ext = namedtuple('mmap', 'addr perms path rss')
+
+    def get_memory_maps(self):
+        try:
+            raw = _psutil_mswindows.get_process_memory_maps(self.pid)
+        except OSError:
+            # XXX - can't use wrap_exceptions decorator as we're
+            # returning a generator; probably needs refactoring.
+            err = sys.exc_info()[1]
+            if err.errno in (errno.EPERM, errno.EACCES, ERROR_ACCESS_DENIED):
+                raise AccessDenied(self.pid, self._process_name)
+            if err.errno == errno.ESRCH:
+                raise NoSuchProcess(self.pid, self._process_name)
+            raise
+        else:
+            for addr, perm, path, rss in raw:
+                path = _convert_raw_path(path)
+                addr = hex(addr)
+                yield (addr, perm, path, rss)
 
     @wrap_exceptions
     def kill_process(self):
@@ -200,14 +235,14 @@ class Process(object):
     @wrap_exceptions
     def get_process_username(self):
         """Return the name of the user that owns the process"""
-        if self.pid in (0, 4) or self.pid == 8 and _WIN2000:
+        if self.pid in (0, 4):
             return 'NT AUTHORITY\\SYSTEM'
-        return _psutil_mswindows.get_process_username(self.pid);
+        return _psutil_mswindows.get_process_username(self.pid)
 
     @wrap_exceptions
     def get_process_create_time(self):
         # special case for kernel process PIDs; return system boot time
-        if self.pid in (0, 4) or self.pid == 8 and _WIN2000:
+        if self.pid in (0, 4):
             return BOOT_TIME
         return _psutil_mswindows.get_process_create_time(self.pid)
 
@@ -220,14 +255,14 @@ class Process(object):
         rawlist = _psutil_mswindows.get_process_threads(self.pid)
         retlist = []
         for thread_id, utime, stime in rawlist:
-            ntuple = ntuple_thread(thread_id, utime, stime)
+            ntuple = nt_thread(thread_id, utime, stime)
             retlist.append(ntuple)
         return retlist
 
     @wrap_exceptions
     def get_cpu_times(self):
         user, system = _psutil_mswindows.get_process_cpu_times(self.pid)
-        return ntuple_cputimes(user, system)
+        return nt_cputimes(user, system)
 
     @wrap_exceptions
     def suspend_process(self):
@@ -239,7 +274,7 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_cwd(self):
-        if self.pid in (0, 4) or self.pid == 8 and _WIN2000:
+        if self.pid in (0, 4):
             raise AccessDenied(self.pid, self._process_name)
         # return a normalized pathname since the native C function appends
         # "\\" at the and of the path
@@ -248,7 +283,7 @@ class Process(object):
 
     @wrap_exceptions
     def get_open_files(self):
-        if self.pid in (0, 4) or self.pid == 8 and _WIN2000:
+        if self.pid in (0, 4):
             return []
         retlist = []
         # Filenames come in in native format like:
@@ -257,15 +292,10 @@ class Process(object):
         # (e.g. "C:\") by using Windows's QueryDosDevice()
         raw_file_names = _psutil_mswindows.get_process_open_files(self.pid)
         for file in raw_file_names:
-            if sys.version_info >= (3,):
-                file = file.decode('utf8')
-            if file.startswith('\\Device\\'):
-                rawdrive = '\\'.join(file.split('\\')[:3])
-                driveletter = _psutil_mswindows.win32_QueryDosDevice(rawdrive)
-                file = file.replace(rawdrive, driveletter)
-                if os.path.isfile(file) and file not in retlist:
-                    ntuple = ntuple_openfile(file, -1)
-                    retlist.append(ntuple)
+            file = _convert_raw_path(file)
+            if os.path.isfile(file) and file not in retlist:
+                ntuple = nt_openfile(file, -1)
+                retlist.append(ntuple)
         return retlist
 
     @wrap_exceptions
@@ -275,7 +305,7 @@ class Process(object):
                              % (kind, ', '.join([repr(x) for x in conn_tmap])))
         families, types = conn_tmap[kind]
         ret = _psutil_mswindows.get_process_connections(self.pid, families, types)
-        return [ntuple_connection(*conn) for conn in ret]
+        return [nt_connection(*conn) for conn in ret]
 
     @wrap_exceptions
     def get_process_nice(self):
@@ -288,7 +318,7 @@ class Process(object):
     @wrap_exceptions
     def get_process_io_counters(self):
         rc, wc, rb, wb =_psutil_mswindows.get_process_io_counters(self.pid)
-        return ntuple_io(rc, wc, rb, wb)
+        return nt_io(rc, wc, rb, wb)
 
     @wrap_exceptions
     def get_process_status(self):
@@ -297,3 +327,36 @@ class Process(object):
             return STATUS_STOPPED
         else:
             return STATUS_RUNNING
+
+    @wrap_exceptions
+    def get_process_cpu_affinity(self):
+        from_bitmask = lambda x: [i for i in xrange(64) if (1 << i) & x]
+        bitmask = _psutil_mswindows.get_process_cpu_affinity(self.pid)
+        return from_bitmask(bitmask)
+
+    @wrap_exceptions
+    def set_process_cpu_affinity(self, value):
+        def to_bitmask(l):
+            if not l:
+                raise ValueError("invalid argument %r" % l)
+            out = 0
+            for b in l:
+                if not isinstance(b, (int, long)) or b < 0:
+                    raise ValueError("invalid argument %r" % b)
+                out |= 2**b
+            return out
+
+        # SetProcessAffinityMask() states that ERROR_INVALID_PARAMETER
+        # is returned for an invalid CPU but this seems not to be true,
+        # therefore we check CPUs validy beforehand.
+        allcpus = list(range(len(get_system_per_cpu_times())))
+        for cpu in value:
+            if cpu not in allcpus:
+                raise ValueError("invalid CPU %i" % cpu)
+
+        bitmask = to_bitmask(value)
+        _psutil_mswindows.set_process_cpu_affinity(self.pid, bitmask)
+
+    @wrap_exceptions
+    def get_num_handles(self):
+        return _psutil_mswindows.get_process_num_handles(self.pid)
