@@ -1,5 +1,5 @@
 /*
- * $Id: _psutil_linux.h 1237 2011-12-14 17:56:08Z g.rodola $
+ * $Id: _psutil_linux.h 1268 2012-03-07 13:28:24Z g.rodola $
  *
  * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
@@ -14,3 +14,6 @@ static PyObject* linux_ioprio_get(PyObject* self, PyObject* args);
 static PyObject* linux_ioprio_set(PyObject* self, PyObject* args);
 static PyObject* get_disk_partitions(PyObject* self, PyObject* args);
 static PyObject* get_physmem(PyObject* self, PyObject* args);
+static PyObject* get_process_cpu_affinity(PyObject* self, PyObject* args);
+static PyObject* set_process_cpu_affinity(PyObject* self, PyObject* args);
+static PyObject* get_system_users(PyObject* self, PyObject* args);
