@@ -1,5 +1,5 @@
 /*
- * $Id: _psutil_osx.c 1229 2011-11-23 22:31:10Z jloden $
+ * $Id: _psutil_osx.c 1360 2012-06-25 22:52:54Z jcscoobyrs@gmail.com $
  *
  * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
@@ -14,6 +14,7 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <utmpx.h>
 #include <sys/sysctl.h>
 #include <sys/vmmeter.h>
 #include <libproc.h>
@@ -21,6 +22,7 @@
 #include <netinet/tcp_fsm.h>
 #include <arpa/inet.h>
 #include <net/if_dl.h>
+#include <pwd.h>
 
 #include <mach/mach.h>
 #include <mach/task.h>
@@ -28,8 +30,11 @@
 #include <mach/host_info.h>
 #include <mach/mach_host.h>
 #include <mach/mach_traps.h>
+#include <mach/mach_vm.h>
 #include <mach/shared_memory_server.h>
 
+#include <mach-o/loader.h>
+
 #include <CoreFoundation/CoreFoundation.h>
 #include <IOKit/IOKitLib.h>
 #include <IOKit/storage/IOBlockStorageDriver.h>
@@ -188,6 +193,131 @@ get_process_tty_nr(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return a list of tuples for every process memory maps.
+ * 'procstat' cmdline utility has been used as an example.
+ */
+static PyObject*
+get_process_memory_maps(PyObject* self, PyObject* args)
+{
+    char buf[PATH_MAX];
+    char addr_str[34];
+    char perms[8];
+    int pagesize = getpagesize();
+    long pid;
+    kern_return_t err = KERN_SUCCESS;
+    mach_port_t task;
+    uint32_t depth = 1;
+    vm_address_t address = 0;
+    vm_size_t size = 0;
+
+    PyObject* py_tuple = NULL;
+    PyObject* py_list = PyList_New(0);
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+
+    err = task_for_pid(mach_task_self(), pid, &task);
+
+    if (err != KERN_SUCCESS) {
+        if (! pid_exists(pid)) {
+            return NoSuchProcess();
+        }
+        // pid exists, so return AccessDenied error since task_for_pid() failed
+        return AccessDenied();
+    }
+
+    while (1) {
+        struct vm_region_submap_info_64 info;
+        mach_msg_type_number_t count = VM_REGION_SUBMAP_INFO_COUNT_64;
+
+        err = vm_region_recurse_64(task, &address, &size, &depth,
+                                   (vm_region_info_64_t)&info, &count);
+
+        if (err == KERN_INVALID_ADDRESS) {
+            break;
+        }
+
+        if (info.is_submap) {
+            depth++;
+        }
+        else {
+            // Free/Reset the char[]s to avoid weird paths
+            memset(buf, 0, sizeof(buf));
+            memset(addr_str, 0, sizeof(addr_str));
+            memset(perms, 0, sizeof(perms));
+
+            sprintf(addr_str, "%016x-%016x", address, address + size);
+            sprintf(perms, "%c%c%c/%c%c%c",
+                    (info.protection & VM_PROT_READ) ? 'r' : '-',
+                    (info.protection & VM_PROT_WRITE) ? 'w' : '-',
+                    (info.protection & VM_PROT_EXECUTE) ? 'x' : '-',
+                    (info.max_protection & VM_PROT_READ) ? 'r' : '-',
+                    (info.max_protection & VM_PROT_WRITE) ? 'w' : '-',
+                    (info.max_protection & VM_PROT_EXECUTE) ? 'x' : '-');
+
+            address += size;
+
+            err = proc_regionfilename(pid, address, buf, sizeof(buf));
+
+            if (info.share_mode == SM_COW && info.ref_count == 1) {
+                // Treat single reference SM_COW as SM_PRIVATE
+                info.share_mode = SM_PRIVATE;
+            }
+
+            if (strlen(buf) == 0) {
+                switch(info.share_mode) {
+                /*
+                case SM_LARGE_PAGE:
+                    // Treat SM_LARGE_PAGE the same as SM_PRIVATE
+                    // since they are not shareable and are wired.
+                */
+                case SM_COW:
+                    strcpy(buf, "[cow]");
+                    break;
+                case SM_PRIVATE:
+                    strcpy(buf, "[prv]");
+                    break;
+                case SM_EMPTY:
+                    strcpy(buf, "[nul]");
+                    break;
+                case SM_SHARED:
+                case SM_TRUESHARED:
+                    strcpy(buf, "[shm]");
+                    break;
+                case SM_PRIVATE_ALIASED:
+                    strcpy(buf, "[ali]");
+                    break;
+                case SM_SHARED_ALIASED:
+                    strcpy(buf, "[s/a]");
+                    break;
+                default:
+                    strcpy(buf, "[???]");
+                }
+            }
+
+            py_tuple = Py_BuildValue("sssIIIIIH",
+                addr_str,                                 // "start-end" address
+                perms,                                    // "rwx" permissions
+                buf,                                      // path
+                info.pages_resident * pagesize,           // rss
+                info.pages_shared_now_private * pagesize, // private
+                info.pages_swapped_out * pagesize,        // swapped
+                info.pages_dirtied * pagesize,            // dirtied
+                info.ref_count,                           // ref count
+                info.shadow_depth                         // shadow depth
+            );
+
+            PyList_Append(py_list, py_tuple);
+            Py_XDECREF(py_tuple);
+        }
+    }
+
+    return py_list;
+}
+
+
 /*
  * Return a Python integer indicating the number of CPUs on the system.
  */
@@ -633,6 +763,8 @@ get_disk_partitions(PyObject* self, PyObject* args)
     int num;
     int i;
     long len;
+    uint64_t flags;
+    char opts[400];
     struct statfs *fs;
     PyObject* py_retlist = PyList_New(0);
     PyObject* py_tuple;
@@ -659,9 +791,65 @@ get_disk_partitions(PyObject* self, PyObject* args)
     }
 
     for (i = 0; i < num; i++) {
-        py_tuple = Py_BuildValue("(sss)", fs[i].f_mntfromname,  // device
-                                          fs[i].f_mntonname,    // mount point
-                                          fs[i].f_fstypename);  // fs type
+        opts[0] = 0;
+        flags = fs[i].f_flags;
+
+        // see sys/mount.h
+        if (flags & MNT_RDONLY)
+            strlcat(opts, "ro", sizeof(opts));
+        else
+            strlcat(opts, "rw", sizeof(opts));
+        if (flags & MNT_SYNCHRONOUS)
+            strlcat(opts, ",sync", sizeof(opts));
+        if (flags & MNT_NOEXEC)
+            strlcat(opts, ",noexec", sizeof(opts));
+        if (flags & MNT_NOSUID)
+            strlcat(opts, ",nosuid", sizeof(opts));
+        if (flags & MNT_UNION)
+            strlcat(opts, ",union", sizeof(opts));
+        if (flags & MNT_ASYNC)
+            strlcat(opts, ",async", sizeof(opts));
+        if (flags & MNT_EXPORTED)
+            strlcat(opts, ",exported", sizeof(opts));
+        if (flags & MNT_QUARANTINE)
+            strlcat(opts, ",quarantine", sizeof(opts));
+        if (flags & MNT_LOCAL)
+            strlcat(opts, ",local", sizeof(opts));
+        if (flags & MNT_QUOTA)
+            strlcat(opts, ",quota", sizeof(opts));
+        if (flags & MNT_ROOTFS)
+            strlcat(opts, ",rootfs", sizeof(opts));
+        if (flags & MNT_DOVOLFS)
+            strlcat(opts, ",dovolfs", sizeof(opts));
+        if (flags & MNT_DONTBROWSE)
+            strlcat(opts, ",dontbrowse", sizeof(opts));
+        if (flags & MNT_IGNORE_OWNERSHIP)
+            strlcat(opts, ",ignore-ownership", sizeof(opts));
+        if (flags & MNT_AUTOMOUNTED)
+            strlcat(opts, ",automounted", sizeof(opts));
+        if (flags & MNT_JOURNALED)
+            strlcat(opts, ",journaled", sizeof(opts));
+        if (flags & MNT_NOUSERXATTR)
+            strlcat(opts, ",nouserxattr", sizeof(opts));
+        if (flags & MNT_DEFWRITE)
+            strlcat(opts, ",defwrite", sizeof(opts));
+        if (flags & MNT_MULTILABEL)
+            strlcat(opts, ",multilabel", sizeof(opts));
+        if (flags & MNT_NOATIME)
+            strlcat(opts, ",noatime", sizeof(opts));
+        if (flags & MNT_UPDATE)
+            strlcat(opts, ",update", sizeof(opts));
+        if (flags & MNT_RELOAD)
+            strlcat(opts, ",reload", sizeof(opts));
+        if (flags & MNT_FORCE)
+            strlcat(opts, ",force", sizeof(opts));
+        if (flags & MNT_CMDFLAGS)
+            strlcat(opts, ",cmdflags", sizeof(opts));
+
+        py_tuple = Py_BuildValue("(ssss)", fs[i].f_mntfromname,  // device
+                                           fs[i].f_mntonname,    // mount point
+                                           fs[i].f_fstypename,   // fs type
+                                           opts);                // options
         PyList_Append(py_retlist, py_tuple);
         Py_XDECREF(py_tuple);
     }
@@ -802,9 +990,8 @@ get_process_open_files(PyObject* self, PyObject* args)
     fds_pointer = malloc(pidinfo_result);
     pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, fds_pointer,
                                   pidinfo_result);
-    free(fds_pointer);
-
     if (pidinfo_result <= 0) {
+        free(fds_pointer);
         goto error;
     }
 
@@ -830,13 +1017,16 @@ get_process_open_files(PyObject* self, PyObject* args)
                     continue;
                 }
                 if (errno != 0) {
+                    free(fds_pointer);
                     return PyErr_SetFromErrno(PyExc_OSError);
                 }
                 else
+                    free(fds_pointer);
                     return PyErr_Format(PyExc_RuntimeError,
                                 "proc_pidinfo(PROC_PIDFDVNODEPATHINFO) failed");
             }
             if (nb < sizeof(vi)) {
+                free(fds_pointer);
                 return PyErr_Format(PyExc_RuntimeError,
                  "proc_pidinfo(PROC_PIDFDVNODEPATHINFO) failed (buffer mismatch)");
             }
@@ -851,6 +1041,7 @@ get_process_open_files(PyObject* self, PyObject* args)
         }
     }
 
+    free(fds_pointer);
     return retList;
 
 error:
@@ -951,9 +1142,9 @@ get_process_connections(PyObject* self, PyObject* args)
     fds_pointer = malloc(pidinfo_result);
     pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, fds_pointer,
                                   pidinfo_result);
-    free(fds_pointer);
 
     if (pidinfo_result <= 0) {
+        free(fds_pointer);
         goto error;
     }
 
@@ -976,14 +1167,17 @@ get_process_connections(PyObject* self, PyObject* args)
                     continue;
                 }
                 if (errno != 0) {
+                    free(fds_pointer);
                     return PyErr_SetFromErrno(PyExc_OSError);
                 }
                 else {
+                    free(fds_pointer);
                     return PyErr_Format(PyExc_RuntimeError,
                                 "proc_pidinfo(PROC_PIDFDVNODEPATHINFO) failed");
                 }
             }
             if (nb < sizeof(si)) {
+                free(fds_pointer);
                 return PyErr_Format(PyExc_RuntimeError,
                  "proc_pidinfo(PROC_PIDFDVNODEPATHINFO) failed (buffer mismatch)");
             }
@@ -1029,6 +1223,7 @@ get_process_connections(PyObject* self, PyObject* args)
             }
 
             if (errno != 0) {
+                free(fds_pointer);
                 return PyErr_SetFromErrno(PyExc_OSError);
             }
 
@@ -1040,7 +1235,7 @@ get_process_connections(PyObject* self, PyObject* args)
                 inet_ntop(AF_INET,
                           &si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_faddr.ina_46.i46a_addr4,
                           rip,
-                          sizeof(lip));
+                          sizeof(rip));
             }
 
             else {
@@ -1049,11 +1244,12 @@ get_process_connections(PyObject* self, PyObject* args)
                           lip, sizeof(lip));
                 inet_ntop(AF_INET6,
                           &si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_faddr.ina_6,
-                          lip, sizeof(rip));
+                          rip, sizeof(rip));
             }
 
             // check for inet_ntop failures
             if (errno != 0) {
+                free(fds_pointer);
                 return PyErr_SetFromErrno(PyExc_OSError);
             }
 
@@ -1085,6 +1281,7 @@ get_process_connections(PyObject* self, PyObject* args)
         }
     }
 
+    free(fds_pointer);
     return retList;
 
 error:
@@ -1103,6 +1300,39 @@ error:
 }
 
 
+/*
+ * Return number of file descriptors opened by process.
+ */
+static PyObject*
+get_process_num_fds(PyObject* self, PyObject* args)
+{
+    long pid;
+    int pidinfo_result;
+    int num;
+    struct proc_fdinfo *fds_pointer;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+
+    pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, NULL, 0);
+    if (pidinfo_result <= 0) {
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+
+    fds_pointer = malloc(pidinfo_result);
+    pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, fds_pointer,
+                                  pidinfo_result);
+    if (pidinfo_result <= 0) {
+        free(fds_pointer);
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+
+    num = (pidinfo_result / PROC_PIDLISTFD_SIZE);
+    return Py_BuildValue("i", num);
+}
+
+
 /*
  * Return a Python list of named tuples with overall network I/O information
  */
@@ -1329,6 +1559,48 @@ get_disk_io_counters(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return currently connected users as a list of tuples.
+ */
+static PyObject*
+get_system_users(PyObject* self, PyObject* args)
+{
+    PyObject *ret_list = PyList_New(0);
+    PyObject *tuple = NULL;
+    struct utmpx ut;
+    FILE *fp;
+
+
+    fp = fopen(_PATH_UTMPX, "r");
+    if (fp == NULL) {
+        // man fopen says errno is set but it seems it's not (OSX 10.6)
+        return PyErr_SetFromErrnoWithFilename(PyExc_OSError, _PATH_UTMPX);
+    }
+
+    while (fread(&ut, sizeof(ut), 1, fp) == 1) {
+        if (*ut.ut_user == '\0') {
+            continue;
+        }
+#ifdef UTMPX_USER_PROCESS
+        if (ut.ut_type != UTMPX_USER_PROCESS) {
+            continue;
+        }
+#endif
+        tuple = Py_BuildValue("(sssf)",
+            ut.ut_user,              // username
+            ut.ut_line,              // tty
+            ut.ut_host,              // hostname
+            (float)ut.ut_tv.tv_sec   // login time
+        );
+        PyList_Append(ret_list, tuple);
+        Py_DECREF(tuple);
+    }
+
+    fclose(fp);
+    return ret_list;
+}
+
+
 /*
  * define the psutil C module methods and initialize the module.
  */
@@ -1362,10 +1634,14 @@ PsutilMethods[] =
          "Return process threads as a list of tuples"},
      {"get_process_open_files", get_process_open_files, METH_VARARGS,
          "Return files opened by process as a list of tuples"},
+     {"get_process_num_fds", get_process_num_fds, METH_VARARGS,
+         "Return the number of fds opened by process."},
      {"get_process_connections", get_process_connections, METH_VARARGS,
          "Get process TCP and UDP connections as a list of tuples"},
      {"get_process_tty_nr", get_process_tty_nr, METH_VARARGS,
          "Return process tty number as an integer"},
+     {"get_process_memory_maps", get_process_memory_maps, METH_VARARGS,
+         "Return a list of tuples for every process's memory map"},
 
      // --- system-related functions
 
@@ -1395,6 +1671,8 @@ PsutilMethods[] =
          "Return dict of tuples of networks I/O information."},
      {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
          "Return dict of tuples of disks I/O information."},
+     {"get_system_users", get_system_users, METH_VARARGS,
+        "Return currently connected users as a list of tuples"},
 
      {NULL, NULL, 0, NULL}
 };
