@@ -1,5 +1,5 @@
 /*
- * $Id: _psutil_osx.h 1142 2011-10-05 18:45:49Z g.rodola $
+ * $Id: _psutil_osx.h 1360 2012-06-25 22:52:54Z jcscoobyrs@gmail.com $
  *
  * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
@@ -24,7 +24,9 @@ static PyObject* get_process_status(PyObject* self, PyObject* args);
 static PyObject* get_process_threads(PyObject* self, PyObject* args);
 static PyObject* get_process_open_files(PyObject* self, PyObject* args);
 static PyObject* get_process_connections(PyObject* self, PyObject* args);
+static PyObject* get_process_num_fds(PyObject* self, PyObject* args);
 static PyObject* get_process_tty_nr(PyObject* self, PyObject* args);
+static PyObject* get_process_memory_maps(PyObject* self, PyObject* args);
 
 // --- system-related functions
 static PyObject* get_pid_list(PyObject* self, PyObject* args);
@@ -39,3 +41,4 @@ static PyObject* get_system_boot_time(PyObject* self, PyObject* args);
 static PyObject* get_disk_partitions(PyObject* self, PyObject* args);
 static PyObject* get_network_io_counters(PyObject* self, PyObject* args);
 static PyObject* get_disk_io_counters(PyObject* self, PyObject* args);
+static PyObject* get_system_users(PyObject* self, PyObject* args);
