@@ -1,5 +1,5 @@
 /*
- * $Id: process_handles.c 1142 2011-10-05 18:45:49Z g.rodola $
+ * $Id: process_handles.c 1295 2012-04-25 01:24:21Z david.daeschler@gmail.com $
  *
  * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
@@ -151,11 +151,14 @@ get_open_files(long pid, HANDLE processHandle)
         handleInfoSize,
         NULL
         )) == STATUS_INFO_LENGTH_MISMATCH)
+    {
         handleInfo = (PSYSTEM_HANDLE_INFORMATION)realloc(handleInfo, handleInfoSize *= 2);
+    }
 
     /* NtQuerySystemInformation stopped giving us STATUS_INFO_LENGTH_MISMATCH. */
     if (!NT_SUCCESS(status)) {
         //printf("NtQuerySystemInformation failed!\n");
+        free(handleInfo);
         return NULL;
     }
 
@@ -207,6 +210,7 @@ get_open_files(long pid, HANDLE processHandle)
             )))
         {
             //printf("[%#x] Error!\n", handle.Handle);
+            free(objectTypeInfo);
             CloseHandle(dupHandle);
             continue;
         }
