@@ -1,5 +1,5 @@
 /*
- * $Id: process_info.c 1142 2011-10-05 18:45:49Z g.rodola $
+ * $Id: process_info.c 1395 2012-06-29 03:42:51Z g.rodola $
  *
  * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
@@ -56,6 +56,11 @@ handle_from_pid_waccess(DWORD pid, DWORD dwDesiredAccess)
     HANDLE hProcess;
     DWORD  processExitCode = 0;
 
+    if (pid == 0) {
+        // otherwise we'd get NoSuchProcess
+        return AccessDenied();
+    }
+
     hProcess = OpenProcess(dwDesiredAccess, FALSE, pid);
     if (hProcess == NULL) {
         if (GetLastError() == ERROR_INVALID_PARAMETER) {
@@ -106,21 +111,21 @@ GetPebAddress(HANDLE ProcessHandle)
 
 DWORD*
 get_pids(DWORD *numberOfReturnedPIDs) {
-    int procArraySz = 1024;
-
     /* Win32 SDK says the only way to know if our process array
      * wasn't large enough is to check the returned size and make
      * sure that it doesn't match the size of the array.
-     * If it does we allocate a larger array and try again*/
+     * If it does we allocate a larger array and try again */
 
-    /* Stores the actual array */
+    // Stores the actual array
     DWORD *procArray = NULL;
     DWORD procArrayByteSz;
+    int procArraySz = 0;
 
-    /* Stores the byte size of the returned array from enumprocesses */
+    // Stores the byte size of the returned array from enumprocesses
     DWORD enumReturnSz = 0;
 
     do {
+        procArraySz += 1024;
         free(procArray);
         procArrayByteSz = procArraySz * sizeof(DWORD);
         procArray = malloc(procArrayByteSz);
@@ -130,16 +135,9 @@ get_pids(DWORD *numberOfReturnedPIDs) {
             PyErr_SetFromWindowsErr(0);
             return NULL;
         }
-        else if (enumReturnSz == procArrayByteSz) {
-            /* Process list was too large.  Allocate more space*/
-            procArraySz += 1024;
-        }
-
-        /* else we have a good list */
-
     } while(enumReturnSz == procArraySz * sizeof(DWORD));
 
-    /* The number of elements is the returned size / size of each element */
+    // The number of elements is the returned size / size of each element
     *numberOfReturnedPIDs = enumReturnSz / sizeof(DWORD);
 
     return procArray;
@@ -277,7 +275,6 @@ get_ppid(long pid)
     if( Process32First(h, &pe)) {
         do {
             if (pe.th32ProcessID == pid) {
-                ////printf("PID: %i; PPID: %i\n", pid, pe.th32ParentProcessID);
                 CloseHandle(h);
                 return Py_BuildValue("I", pe.th32ParentProcessID);
             }
@@ -294,7 +291,6 @@ get_ppid(long pid)
 }
 
 
-
 /*
  * returns a Python list representing the arguments for the process
  * with given pid or NULL on error.
@@ -447,6 +443,7 @@ get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess, PVOID *retB
     hNtDll = LoadLibrary(TEXT("ntdll.dll"));
     NtQuerySystemInformation = (NTQSI_PROC)GetProcAddress(
                                 hNtDll, "NtQuerySystemInformation");
+    FreeLibrary(hNtDll);
 
     bufferSize = initialBufferSize;
     buffer = malloc(bufferSize);
@@ -467,6 +464,8 @@ get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess, PVOID *retB
 
     if (status != 0) {
         PyErr_Format(PyExc_RuntimeError, "NtQuerySystemInformation() failed");
+
+        free(buffer);
         return 0;
     }
 
@@ -484,7 +483,7 @@ get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess, PVOID *retB
     } while ( (process = PH_NEXT_PROCESS(process)) );
 
     NoSuchProcess();
+
+    free(buffer);
     return 0;
 }
-
-
