@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: _osx.py 1142 2011-10-05 18:45:49Z g.rodola $
+# $Id: _osx.py 1335 2012-06-03 18:21:54Z g.rodola $
 #
 # Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -12,11 +12,12 @@ import unittest
 import subprocess
 import time
 import sys
+import os
 
 import psutil
 
+from psutil._compat import PY3
 from test_psutil import reap_children, get_test_subprocess, sh
-#from _posix import ps
 
 
 def sysctl(cmdline):
@@ -25,7 +26,7 @@ def sysctl(cmdline):
     """
     p = subprocess.Popen(cmdline, shell=1, stdout=subprocess.PIPE)
     result = p.communicate()[0].strip().split()[1]
-    if sys.version_info >= (3,):
+    if PY3:
         result = str(result, sys.stdout.encoding)
     try:
         return int(result)
@@ -45,11 +46,16 @@ class OSXSpecificTestCase(unittest.TestCase):
         sysctl_hwphymem = sysctl('sysctl hw.memsize')
         self.assertEqual(sysctl_hwphymem, psutil.TOTAL_PHYMEM)
 
+    def test_total_virtmem(self):
+        tot1 = psutil.virtmem_usage().total
+        tot2 = os.path.getsize("/var/vm/swapfile0")
+        self.assertEqual(tot1, tot2)
+
     def test_process_create_time(self):
         cmdline = "ps -o lstart -p %s" %self.pid
         p = subprocess.Popen(cmdline, shell=1, stdout=subprocess.PIPE)
         output = p.communicate()[0]
-        if sys.version_info >= (3,):
+        if PY3:
             output = str(output, sys.stdout.encoding)
         start_ps = output.replace('STARTED', '').strip()
         start_psutil = psutil.Process(self.pid).create_time
