@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: _windows.py 1203 2011-10-24 19:10:36Z g.rodola $
+# $Id: _windows.py 1394 2012-06-29 03:21:43Z g.rodola $
 #
 # Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -17,10 +17,12 @@ import warnings
 import atexit
 import sys
 import subprocess
+import errno
 
 import psutil
 import _psutil_mswindows
-from test_psutil import reap_children, get_test_subprocess, wait_for_pid, PY3
+from psutil._compat import PY3, callable
+from test_psutil import reap_children, get_test_subprocess, wait_for_pid
 try:
     import wmi
 except ImportError:
@@ -28,8 +30,15 @@ except ImportError:
     atexit.register(warnings.warn, "Couldn't run wmi tests: %s" % str(err),
                     RuntimeWarning)
     wmi = None
+try:
+    import win32api
+    import win32con
+except ImportError:
+    err = sys.exc_info()[1]
+    atexit.register(warnings.warn, "Couldn't run pywin32 tests: %s" % str(err),
+                    RuntimeWarning)
+    win32api = None
 
-WIN2000 = platform.win32_ver()[0] == '2000'
 
 
 class WindowsSpecificTestCase(unittest.TestCase):
@@ -47,10 +56,7 @@ class WindowsSpecificTestCase(unittest.TestCase):
         self.assertRaises(psutil.AccessDenied, p.kill)
 
     def test_special_pid(self):
-        if not WIN2000:
-            p = psutil.Process(4)
-        else:
-            p = psutil.Process(8)
+        p = psutil.Process(4)
         self.assertEqual(p.name, 'System')
         # use __str__ to access all common Process properties to check
         # that nothing strange happens
@@ -65,13 +71,11 @@ class WindowsSpecificTestCase(unittest.TestCase):
                 raise
         else:
             self.assertTrue(rss > 0)
-            if not WIN2000:
-                self.assertEqual(vms, 0)
 
     def test_signal(self):
         p = psutil.Process(self.pid)
         self.assertRaises(ValueError, p.send_signal, signal.SIGINT)
-        
+
     def test_nic_names(self):
         p = subprocess.Popen(['ipconfig', '/all'], stdout=subprocess.PIPE)
         out = p.communicate()[0]
@@ -84,6 +88,13 @@ class WindowsSpecificTestCase(unittest.TestCase):
             if nic not in out:
                 self.fail("%r nic wasn't found in 'ipconfig /all' output" % nic)
 
+    def test_exe(self):
+        for p in psutil.process_iter():
+            try:
+                self.assertEqual(os.path.basename(p.exe), p.name)
+            except psutil.Error:
+                pass
+
     if wmi is not None:
 
         # --- Process class tests
@@ -98,11 +109,10 @@ class WindowsSpecificTestCase(unittest.TestCase):
             p = psutil.Process(self.pid)
             self.assertEqual(p.exe, w.ExecutablePath)
 
-        if not WIN2000:
-            def test_process_cmdline(self):
-                w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
-                p = psutil.Process(self.pid)
-                self.assertEqual(' '.join(p.cmdline), w.CommandLine.replace('"', ''))
+        def test_process_cmdline(self):
+            w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
+            p = psutil.Process(self.pid)
+            self.assertEqual(' '.join(p.cmdline), w.CommandLine.replace('"', ''))
 
         def test_process_username(self):
             w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
@@ -183,7 +193,15 @@ class WindowsSpecificTestCase(unittest.TestCase):
                         if not ps_part.mountpoint:
                             # this is usually a CD-ROM with no disk inserted
                             break
-                        usage = psutil.disk_usage(ps_part.mountpoint)
+                        try:
+                            usage = psutil.disk_usage(ps_part.mountpoint)
+                        except OSError:
+                            err = sys.exc_info()[1]
+                            if err.errno == errno.ENOENT:
+                                # usually this is the floppy
+                                break
+                            else:
+                                raise
                         self.assertEqual(usage.total, int(wmi_part.Size))
                         wmi_free = int(wmi_part.FreeSpace)
                         self.assertEqual(usage.free, wmi_free)
@@ -192,11 +210,57 @@ class WindowsSpecificTestCase(unittest.TestCase):
                             self.fail("psutil=%s, wmi=%s" % usage.free, wmi_free)
                         break
                 else:
-                    self.fail("can't find partition %r", ps_part)
+                    self.fail("can't find partition %s" % repr(ps_part))
+
+    if win32api is not None:
+
+        def test_get_num_handles(self):
+            p = psutil.Process(os.getpid())
+            before = p.get_num_handles()
+            handle = win32api.OpenProcess(win32con.PROCESS_QUERY_INFORMATION,
+                                          win32con.FALSE, os.getpid())
+            after = p.get_num_handles()
+            self.assertEqual(after, before+1)
+            win32api.CloseHandle(handle)
+            self.assertEqual(p.get_num_handles(), before)
+
+        def test_get_num_handles_2(self):
+            # Note: this fails from time to time; I'm keen on thinking
+            # it doesn't mean something is broken
+            def call(p, attr):
+                attr = getattr(p, name, None)
+                if attr is not None and callable(attr):
+                    ret = attr()
+                else:
+                    ret = attr
+
+            p = psutil.Process(self.pid)
+            attrs = []
+            failures = []
+            for name in dir(psutil.Process):
+                if name.startswith('_') \
+                or name.startswith('set_') \
+                or name in ('terminate', 'kill', 'suspend', 'resume', 'nice',
+                            'send_signal', 'wait', 'get_children'):
+                    continue
+                else:
+                    try:
+                        call(p, name)
+                        num1 = p.get_num_handles()
+                        call(p, name)
+                        num2 = p.get_num_handles()
+                    except (psutil.NoSuchProcess, psutil.AccessDenied):
+                        pass
+                    else:
+                        if num2 > num1:
+                            fail = "failure while processing Process.%s method " \
+                                   "(before=%s, after=%s)" % (name, num1, num2)
+                            failures.append(fail)
+            if failures:
+                self.fail('\n' + '\n'.join(failures))
 
 
 if __name__ == '__main__':
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(WindowsSpecificTestCase))
     unittest.TextTestRunner(verbosity=2).run(test_suite)
-
